#    wview is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    wview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.

#    author: Federico Salotti
#    company/organization: MR_ROBOT_COMPUTERS_REPAIR
#    cuntry: Italy
#    city: Barga(LU)
#    website: mrrobotcomputers.hopto.org
#    mail: mrrobotcomputersrepair@gmail.com
#    release: 1.1 (15 February 2021 00:35)

import numpy
import cv2
import imutils
from imutils import paths
import random
import time
import argparse
import animations


parser=argparse.ArgumentParser()
parser.add_argument("-name",help="set name of the window") # nome finestra
parser.add_argument("-width",type=int,help="set initial width of the window") # cam width
parser.add_argument("-height",type=int,help="set initial height of the window") # cam height
parser.add_argument("-delay",type=int,help="set delay time in seconds between animations") # set delay value
parser.add_argument("-nodelay", action="store_true",help="no delay between animations") # nessuna pausa tra una animazione e l'altra
parser.add_argument("-nomirror", action="store_true",help="disable cam mirroring") # mirroring
parser.add_argument("-noanimation", action="store_true",help="disable cam animations") # animationless

args=parser.parse_args()
animation_end=True
animation_less = False

if args.noanimation:
	animation_less = True


def main():

	animation_id=0
	latest_animation_id=0
	start_tick=time.time()
	tick=[0]
	animation_index=[0]

	### STATIC SPRITE SPAWN ###
	sprite_y=[0]
	sprite_x=[100]
	sprite_randYX=[True]
	spawn_points=[]
	y_ax=[False]
	x_ax=[False]

	### PACMAN ANIMATION DEDICATED ###

	pac_animation_index=[0]
#########################################
	blinky_x=[60]
	blinky_y=[0]
	blinky_remove=[0]
########################################

	inky_x=[60]
	inky_y=[0]
	inky_remove=[0]

######################################

	pinky_x=[60]
	pinky_y=[0]
	pinky_remove=[0]

######################################

	clyde_x=[60]
	clyde_y=[0]
	clyde_remove=[0]
#######################################

	### END PACMAN ANIMATION DEDICATED ###
	global window_name
	window_name="wview 1.0"
	delay=5

	if args.delay:
		delay = args.delay

	if args.name:
		window_name=args.name

	if args.nodelay:
		delay=0


	cv2.namedWindow(window_name,cv2.WINDOW_GUI_NORMAL)
	cv2.resizeWindow(window_name, 400,400)

	y=[0]
	x=[100]
	inverse_y=[540]
	inverse_x=[640]

	cam=cv2.VideoCapture(0)

	if args.width and args.height:
		cam.set(cv2.CAP_PROP_FRAME_WIDTH,args.width)
		cam.set(cv2.CAP_PROP_FRAME_HEIGHT,args.height)

	elif args.width and not args.height:
		cam.set(cv2.CAP_PROP_FRAME_WIDTH,args.width)
		cam.set(cv2.CAP_PROP_FRAME_HEIGHT,args.width)

	elif not args.width and args.height:
		cam.set(cv2.CAP_PROP_FRAME_WIDTH,args.height)
		cam.set(cv2.CAP_PROP_FRAME_HEIGHT,args.height)

	animations.init(args,cv2,window_name,random)


	while True:

		if animation_id==0 and animation_end==True:
			animation_id=random.randint(0,12)
			while animation_id==latest_animation_id:
				if animation_id != latest_animation_id:
					animation_id=animation_id
					break
				else:
					animation_id=random.randint(0,11)
		else:
			pass

		val,frame=cam.read()
		if animation_id==1 and  animation_end==True and tick[0]>int(delay) and not animation_less:
			if animations.pacman_animation(frame,x,y,tick,pac_animation_index,blinky_x,blinky_y,blinky_remove,inky_x,inky_y,inky_remove,pinky_x,pinky_y,pinky_remove,clyde_x,clyde_y,clyde_remove):
				start_tick=time.time()
				animation_id=0
				latest_animation_id=1

		elif animation_id==2 and animation_end==True and tick[0]>int(delay) and not animation_less:
			if animations.pokeball_animation(frame,x,y,tick,animation_index,sprite_randYX,sprite_y,sprite_x,spawn_points):
				start_tick=time.time()
				animation_id=0
				latest_animation_id=2


		elif animation_id==3 and animation_end==True and tick[0]>int(delay) and not animation_less:
			if animations.mario_fly_animation(frame,x,y,tick):
				start_tick=time.time()
				animation_id=0
				latest_animation_id=3


		elif animation_id==4 and animation_end==True and tick[0]>int(delay) and not animation_less:
			if animations.goomba_main(frame,x,y,tick,animation_index):
				start_tick=time.time()
				animation_id=0
				latest_animation_id=4


		elif animation_id==5 and animation_end==True and tick[0]>int(delay) and not animation_less:
			if animations.mario_run(frame,x,y,tick,animation_index):
				start_tick=time.time()
				animation_id=0
				latest_animation_id=5


		elif animation_id==6 and animation_end==True and tick[0]>int(delay) and not animation_less:
			if animations.charizard_fly(frame,x,y,tick,animation_index,y_ax,x_ax,spawn_points):
				start_tick=time.time()
				animation_id=0
				latest_animation_id=6


		elif animation_id==7 and animation_end==True and tick[0]>int(delay) and not animation_less:
			if animations.arthur(frame,x,y,tick,animation_index):
				start_tick=time.time()
				animation_id=0
				latest_animation_id=7

		elif animation_id==8 and animation_end==True and tick[0]>int(delay) and not animation_less:
			if animations.sonic(frame,x,y,tick,animation_index):
				start_tick=time.time()
				animation_id=0
				latest_animation_id=8

		elif animation_id==9 and animation_end==True and tick[0]>int(delay) and not animation_less:
			if animations.ninja_gaiden(frame,x,y,tick,animation_index):
				start_tick=time.time()
				animation_id=0
				latest_animation_id=9

		elif animation_id==10 and animation_end==True and tick[0]>int(delay) and not animation_less:
			if animations.kirby_umbrella(frame,x,y,tick,animation_index,sprite_randYX,spawn_points):
				start_tick=time.time()
				animation_id=0
				latest_animation_id=10

		elif animation_id==11 and animation_end==True and tick[0]>int(delay) and not animation_less:
			if animations.tails(frame,inverse_x,inverse_y,tick,animation_index,y_ax,x_ax,spawn_points):
				start_tick=time.time()
				animation_id=0
				latest_animation_id=11

		elif animation_id==12 and animation_end==True and tick[0]>int(delay) and not animation_less:
			if animations.tux(frame,x,y,tick,animation_index,sprite_randYX,sprite_y,sprite_x,spawn_points):
				start_tick=time.time()
				animation_id=0
				latest_animation_id=12


		else:
			animations.no_animation(frame)

		if cv2.waitKey(1)==27:
			break


		tick[0]=round((time.time()-start_tick),2)
		#print (delay)

main()
